@@ -35,21 +35,21 @@ class Rubinius::VM
   # Used to support error reporting where IO is not reliable yet.
   #
   def self.write_error(str)
-    Ruby.primitive :vm_write_error
+    Rubinius.primitive :vm_write_error
     raise PrimitiveFailure, "Rubinius::VM.write_error primitive failed"
   end
 
   # Prints Ruby backtrace at point of call.
   #
   def self.show_backtrace
-    Ruby.primitive :vm_show_backtrace
+    Rubinius.primitive :vm_show_backtrace
     raise PrimitiveFailure, "Rubinius::VM.show_backtrace primitive failed"
   end
 
   # Reset the method cache globally for given method name.
   #
-  def self.reset_method_cache(sym)
-    Ruby.primitive :vm_reset_method_cache
+  def self.reset_method_cache(mod, sym)
+    Rubinius.primitive :vm_reset_method_cache
     raise PrimitiveFailure, "Rubinius::VM.reset_method_cache primitive failed"
   end
 end
@@ -59,7 +59,7 @@ class Object
   # Prints basic information about the object to stdout.
   #
   def __show__
-    Ruby.primitive :object_show
+    Rubinius.primitive :object_show
     "<unknown>"
   end
 end
@@ -76,7 +76,7 @@ class Class
   # See .new
   #
   def allocate
-    Ruby.primitive :class_allocate
+    Rubinius.primitive :class_allocate
     raise RuntimeError, "Class#allocate primitive failed on #{self.inspect}"
   end
 
@@ -100,7 +100,7 @@ class Class
       # allowed to leak a stack value
     end
 
-    return obj
+    obj
   end
 end
 
@@ -113,7 +113,7 @@ module Kernel
   # explicit receiver, since class by itself is a keyword.
   #
   def class
-    Ruby.primitive :object_class
+    Rubinius.primitive :object_class
     raise PrimitiveFailure, "Kernel#class primitive failed."
   end
 
@@ -121,7 +121,7 @@ module Kernel
   #
   # This is the same as #class, but named to always be available.
   def __class__
-    Ruby.primitive :object_class
+    Rubinius.primitive :object_class
     raise PrimitiveFailure, "Kernel#class primitive failed."
   end
 
@@ -157,8 +157,8 @@ module Kernel
   # If the argument is not a Class or Module, a TypeError is raised.
   #
   def kind_of?(cls)
-    Ruby.primitive :object_kind_of
-    raise TypeError, 'kind_of? requires a Class or Module argument'
+    Rubinius.primitive :object_kind_of
+    raise TypeError, "Kernel#kind_of? requires a Class or Module argument"
   end
 
   # Hook method invoked when object is sent a message it cannot handle.
@@ -169,7 +169,7 @@ module Kernel
   # This method may be overridden, and is often used to provide dynamic
   # behaviour. An overriding version should call super if it fails to
   # resolve the message. This practice ensures that the default version
-  # will called if all else fails.
+  # will be called if all else fails.
   #
   def method_missing(meth, *args)
     raise NoMethodError, "Unable to send '#{meth}' on '#{self}' (#{self.class})"
@@ -200,14 +200,11 @@ module Kernel
   # if defined.
   #
   def dup
-    cls = Rubinius.invoke_primitive :object_class, self
-    copy = cls.allocate
+    copy = Rubinius::Type.object_class(self).allocate
 
     Rubinius.invoke_primitive :object_copy_object, copy, self
 
-    Rubinius.privately do
-      copy.initialize_copy self
-    end
+    Rubinius::Type.object_initialize_dup self, copy
     copy
   end
 
@@ -229,15 +226,12 @@ module Kernel
   def clone
     # Do not implement in terms of dup. It breaks rails.
     #
-    cls = Rubinius.invoke_primitive :object_class, self
-    copy = cls.allocate
+    copy = Rubinius::Type.object_class(self).allocate
 
     Rubinius.invoke_primitive :object_copy_object, copy, self
     Rubinius.invoke_primitive :object_copy_singleton_class, copy, self
 
-    Rubinius.privately do
-      copy.initialize_copy self
-    end
+    Rubinius::Type.object_initialize_clone self, copy
 
     copy.freeze if frozen?
     copy
@@ -258,7 +252,7 @@ module Rubinius
     # Specialised allocation.
     #
     def self.allocate
-      Ruby.primitive :accessvariable_allocate
+      Rubinius.primitive :accessvariable_allocate
       raise PrimitiveFailure, "AccessVariable.allocate primitive failed"
     end
 
@@ -303,18 +297,40 @@ module Rubinius
     # Retrieve value for given key.
     #
     def [](key)
-      Ruby.primitive :lookuptable_aref
+      Rubinius.primitive :lookuptable_aref
       raise PrimitiveFailure, "LookupTable#[] primitive failed"
     end
 
     # Store value under key.
     #
     def []=(key, val)
-      Ruby.primitive :lookuptable_store
+      Rubinius.primitive :lookuptable_store
       raise PrimitiveFailure, "LookupTable#[]= primitive failed"
     end
   end
 
+  # Constant table for storing methods.
+  #
+  # See kernel/bootstrap/constant_table.rb and
+  #     kernel/common/constant_table.rb
+  #
+  class ConstantTable
+
+    # Perform lookup for constant name.
+    #
+    def lookup(name)
+      Rubinius.primitive :constant_table_lookup
+      raise PrimitiveFailure, "ConstantTable#lookup primitive failed"
+    end
+
+    # Store Constant under name, with given visibility.
+    #
+    def store(name, constant, visibility)
+      Rubinius.primitive :constant_table_store
+      raise PrimitiveFailure, "ConstantTable#store primitive failed"
+    end
+  end
+
   # Lookup table for storing methods.
   #
   # See kernel/bootstrap/methodtable.rb and
@@ -325,21 +341,21 @@ module Rubinius
     # Perform lookup for method name.
     #
     def lookup(name)
-      Ruby.primitive :methodtable_lookup
+      Rubinius.primitive :methodtable_lookup
       raise PrimitiveFailure, "MethodTable#lookup primitive failed"
     end
 
     # Store Executable under name, with given visibility.
     #
     def store(name, exec, visibility)
-      Ruby.primitive :methodtable_store
+      Rubinius.primitive :methodtable_store
       raise PrimitiveFailure, "MethodTable#store primitive failed"
     end
 
     # Make an alias from +original_name+ in +original_mod+ to +name+
     # with visibility +vis+
     def alias(name, visibility, original_name, original_exec, original_mod)
-      Ruby.primitive :methodtable_alias
+      Rubinius.primitive :methodtable_alias
       raise PrimitiveFailure, "MethodTable#alias primitive failed"
     end
   end
@@ -350,8 +366,8 @@ class Symbol
   # Produce String representation of this Symbol.
   #
   def to_s
-    Ruby.primitive :symbol_to_s
-    raise PrimitiveFailure, "Symbol#to_s primitive failed."
+    Rubinius.primitive :symbol_to_s
+    raise PrimitiveFailure, "Symbol#to_s primitive failed"
   end
 
   # For completeness, returns self.
@@ -380,8 +396,8 @@ class String
   # TODO: Add taintedness-check
   #++
   def to_sym
-    Ruby.primitive :symbol_lookup
-    raise PrimitiveFailure, "Unable to symbolize: #{self.dump}"
+    Rubinius.primitive :symbol_lookup
+    raise PrimitiveFailure, "String#to_sym primitive failed: #{self.dump}"
   end
 
   # For completeness, returns self.
@@ -409,7 +425,7 @@ module Process
   end
 
   def self.exit!(code=1)
-    Ruby.primitive :vm_exit
+    Rubinius.primitive :vm_exit
 
     case code
     when true
@@ -425,13 +441,14 @@ end
 
 class Module
   def method_table   ; @method_table ; end
-  def constant_table; @constant_table ; end
+  def constant_table ; @constant_table ; end
   def name           ; @module_name.to_s ; end
+  def origin         ; @origin ; end
 
   # Specialised allocator.
   #
   def self.allocate
-    Ruby.primitive :module_allocate
+    Rubinius.primitive :module_allocate
     raise PrimitiveFailure, "Module.allocate primitive failed"
   end
 
@@ -452,11 +469,12 @@ class Module
 
   # Set Module's direct superclass.
   #
-  # The corresponding 'getter' #superclass method defined
+  # The corresponding 'getter' #superclass method is defined
   # in class.rb, because it is more complex than a mere
   # accessor
   #
   def superclass=(other)
+    Rubinius.check_frozen
     @superclass = other
   end
 
@@ -489,7 +507,7 @@ class Module
   #
   # Basic version of .include used in kernel code.
   #
-  # Redefined in kernel/common/module.rb.
+  # Redefined in kernel/delta/module.rb.
   #
   def include(mod)
     mod.append_features(self)
@@ -508,14 +526,14 @@ class Module
   def attr_reader(name)
     meth = Rubinius::AccessVariable.get_ivar name
     @method_table.store name, meth, :public
-    Rubinius::VM.reset_method_cache name
+    Rubinius::VM.reset_method_cache self, name
     nil
   end
 
   def attr_reader_specific(name, method_name)
     meth = Rubinius::AccessVariable.get_ivar name
     @method_table.store method_name, meth, :public
-    Rubinius::VM.reset_method_cache method_name
+    Rubinius::VM.reset_method_cache self, method_name
     nil
   end
 
@@ -527,8 +545,9 @@ class Module
   #
   def attr_writer(name)
     meth = Rubinius::AccessVariable.set_ivar name
-    @method_table.store "#{name}=".to_sym, meth, :public
-    Rubinius::VM.reset_method_cache name
+    writer_name = "#{name}=".to_sym
+    @method_table.store writer_name, meth, :public
+    Rubinius::VM.reset_method_cache self, writer_name
     nil
   end
 
@@ -550,7 +569,21 @@ class Module
   # Cannot be used as a toggle, and only
   # takes a single method name.
   #
-  # Redefined in kernel/common/module.rb.
+  # Redefined in kernel/delta/module.rb.
+  #
+  def public(name)
+    if entry = @method_table.lookup(name)
+      entry.visibility = :public
+    end
+  end
+
+  # :internal:
+  #
+  # Basic version used in kernel code.
+  # Cannot be used as a toggle, and only
+  # takes a single method name.
+  #
+  # Redefined in kernel/delta/module.rb.
   #
   def private(name)
     if entry = @method_table.lookup(name)
@@ -578,7 +611,7 @@ class Module
   # of current method and stores it under the new name.
   # The two are independent.
   #
-  # Redefined in kernel/common/module.rb.
+  # Redefined in kernel/delta/module.rb.
   #
   def alias_method(new_name, current_name)
     # If we're aliasing a method we contain, just reference it directly, no
@@ -601,7 +634,7 @@ class Module
                           entry.method, mod
     end
 
-    Rubinius::VM.reset_method_cache(new_name)
+    Rubinius::VM.reset_method_cache self, new_name
   end
 
   # :internal:
@@ -615,9 +648,15 @@ class Module
     if entry = @method_table.lookup(name)
       sc = class << self; self; end
       sc.method_table.store name, entry.method, :public
+      Rubinius::VM.reset_method_cache self, name
       private name
     end
   end
+
+  def track_subclass(cls)
+    Rubinius.primitive :module_track_subclass
+    raise PrimitiveFailure, "Module.track_subclass primitive failed"
+  end
 end
 
 
@@ -668,7 +707,7 @@ module Rubinius
     # Specialised allocator.
     #
     def self.allocate
-      Ruby.primitive :included_module_allocate
+      Rubinius.primitive :included_module_allocate
       raise PrimitiveFailure, "IncludedModule.allocate primitive failed"
     end
 
@@ -694,6 +733,8 @@ module Rubinius
     def attach_to(cls)
       @superclass = cls.direct_superclass
       cls.superclass = self
+      @module.track_subclass(cls)
+      self
     end
 
     # :internal:
@@ -732,3 +773,4 @@ end
 class Object
   include Kernel
 end
+
