@@ -6,13 +6,13 @@ module Rubinius
     private :initialize
 
     def get_variable(name)
-      Ruby.primitive :vm_get_config_item
-      raise PrimitiveFailure, "Unable to get config variable"
+      Rubinius.primitive :vm_get_config_item
+      raise PrimitiveFailure, "Rubinius::Configuration#get_variable primitive failed"
     end
 
     def get_section(section)
-      Ruby.primitive :vm_get_config_section
-      raise PrimitiveFailure, "Unable to get config section"
+      Rubinius.primitive :vm_get_config_section
+      raise PrimitiveFailure, "Rubinius::Configuration#get_section primitive failed"
     end
 
     def section(section)
