@@ -5,6 +5,9 @@ module Enumerable
     attr_writer :args
     private :args=
 
+    attr_writer :size
+    private :size=
+
     def initialize_enumerator(receiver, size, method_name, *method_args)
       @object = receiver
       @size = size
@@ -12,6 +15,7 @@ module Enumerable
       @args = method_args
       @generator = nil
       @lookahead = []
+      @feedvalue = nil
 
       self
     end
@@ -68,7 +72,12 @@ module Enumerable
 
     def each_with_block
       @object.__send__ @iter, *@args do |*args|
-        yield(*args)
+        ret = yield(*args)
+        unless @feedvalue.nil?
+          ret = @feedvalue
+          @feedvalue = nil
+        end
+        ret
       end
     end
     private :each_with_block
@@ -159,6 +168,12 @@ module Enumerable
       end
     end
 
+    def feed(val)
+      raise TypeError, "Feed value already set" unless @feedvalue.nil?
+      @feedvalue = val
+      nil
+    end
+
     class Yielder
       def initialize(&block)
         raise LocalJumpError, "Expected a block to be given" unless block_given?
