@@ -1,3 +1,4 @@
+module Rubinius
 module FFI::Platform::POSIX
   #--
   # Internal class for accessing timevals
@@ -6,6 +7,7 @@ module FFI::Platform::POSIX
     config 'rbx.platform.timeval', :tv_sec, :tv_usec
   end
 end
+end
 
 class File < IO
   include Enumerable
@@ -22,6 +24,10 @@ class File < IO
     # O_ACCMODE is /undocumented/ for fcntl() on some platforms
     ACCMODE  = Rubinius::Config['rbx.platform.fcntl.O_ACCMODE']
 
+    F_GETFD  = Rubinius::Config['rbx.platform.fcntl.F_GETFD']
+    F_SETFD  = Rubinius::Config['rbx.platform.fcntl.F_SETFD']
+    FD_CLOEXEC = Rubinius::Config['rbx.platform.fcntl.FD_CLOEXEC']
+
     RDONLY   = Rubinius::Config['rbx.platform.file.O_RDONLY']
     WRONLY   = Rubinius::Config['rbx.platform.file.O_WRONLY']
     RDWR     = Rubinius::Config['rbx.platform.file.O_RDWR']
@@ -51,8 +57,16 @@ class File < IO
     FNM_PATHNAME = 0x02
     FNM_DOTMATCH = 0x04
     FNM_CASEFOLD = 0x08
+
+    if Rubinius.windows?
+      NULL = 'NUL'
+    else
+      NULL = '/dev/null'
+    end
   end
 
+  FFI = Rubinius::FFI
+
   SEPARATOR = FFI::Platform::File::SEPARATOR
   Separator = FFI::Platform::File::SEPARATOR
   ALT_SEPARATOR = FFI::Platform::File::ALT_SEPARATOR
@@ -61,33 +75,6 @@ class File < IO
 
   attr_reader :path
 
-  def initialize(path_or_fd, mode = "r", perm = 0666)
-    if path_or_fd.kind_of? Integer
-      super(path_or_fd, mode)
-      @path = nil
-    else
-      path = StringValue(path_or_fd)
-
-      fd = IO.sysopen(path, mode, perm)
-      if fd < 0
-        begin
-          Errno.handle path
-        rescue Errno::EMFILE
-          # true means force to run, don't ignore it.
-          GC.run(true)
-
-          fd = IO.sysopen(path, mode, perm)
-          Errno.handle if fd < 0
-        end
-      end
-
-      @path = path
-      super(fd)
-    end
-  end
-
-  private :initialize
-
   # The mode_t type is 2 bytes (ushort). Instead of getting whatever
   # value happens to be in the least significant 16 bits, just set
   # the value to 0 if it is greater than 0xffff. Also, negative values
@@ -98,6 +85,15 @@ class File < IO
   end
   module_function :clamp_short
 
+  def self.absolute_path(obj, dir = nil)
+    obj = path(obj)
+    if obj[0] == "~"
+      File.join Dir.getwd, dir.to_s, obj
+    else
+      expand_path(obj, dir)
+    end
+  end
+
   ##
   # Returns the last access time for the named file as a Time object).
   #
@@ -115,23 +111,23 @@ class File < IO
   #
   #  File.basename("/home/gumby/work/ruby.rb")          #=> "ruby.rb"
   #  File.basename("/home/gumby/work/ruby.rb", ".rb")   #=> "ruby"
-  def self.basename(path,ext=undefined)
-    path = StringValue(path)
+  def self.basename(path, ext=undefined)
+    path = Rubinius::Type.coerce_to_path(path)
 
     slash = "/"
 
-    ext_not_present = ext.equal?(undefined)
+    ext_not_present = undefined.equal?(ext)
 
-    if pos = path.find_string_reverse(slash, path.size)
+    if pos = path.find_string_reverse(slash, path.bytesize)
       # special case. If the string ends with a /, ignore it.
-      if pos == path.size - 1
+      if pos == path.bytesize - 1
 
         # Find the first non-/ from the right
         data = path.data
         found = false
         pos.downto(0) do |i|
-          if data[i] != ?/
-            path = path.substring(0, i+1)
+          if data[i] != 47  # ?/
+            path = path.byteslice(0, i+1)
             found = true
             break
           end
@@ -141,14 +137,14 @@ class File < IO
         return slash unless found
 
         # Now that we've trimmed the /'s at the end, search again
-        pos = path.find_string_reverse(slash, path.size)
+        pos = path.find_string_reverse(slash, path.bytesize)
         if ext_not_present and !pos
           # No /'s found and ext not present, return path.
           return path
         end
       end
 
-      path = path.substring(pos + 1, path.size - pos) if pos
+      path = path.byteslice(pos + 1, path.bytesize - pos) if pos
     end
 
     return path if ext_not_present
@@ -158,13 +154,13 @@ class File < IO
     ext = StringValue(ext)
 
     if ext == ".*"
-      if pos = path.rindex(?.)
-        return path.substring(0, pos)
+      if pos = path.find_string_reverse(".", path.bytesize)
+        return path.byteslice(0, pos)
       end
-    elsif pos = path.rindex(ext)
+    elsif pos = path.find_string_reverse(ext, path.bytesize)
       # Check that ext is the last thing in the string
-      if pos == path.size - ext.size
-        return path.substring(0, pos)
+      if pos == path.bytesize - ext.size
+        return path.byteslice(0, pos)
       end
     end
 
@@ -197,7 +193,8 @@ class File < IO
     mode = clamp_short mode
 
     paths.each do |path|
-      POSIX.chmod StringValue(path), mode
+      n = POSIX.chmod Rubinius::Type.coerce_to_path(path), mode
+      Errno.handle if n == -1
     end
     paths.size
   end
@@ -208,10 +205,13 @@ class File < IO
   # the link, not the file referenced by the link).
   # Often not available.
   def self.lchmod(mode, *paths)
+    raise NotImplementedError, "lchmod not implemented on this platform" unless Rubinius::HAVE_LCHMOD
+
     mode = Rubinius::Type.coerce_to(mode, Integer, :to_int)
 
     paths.each do |path|
-      POSIX.lchmod StringValue(path), mode
+      n = POSIX.lchmod Rubinius::Type.coerce_to_path(path), mode
+      Errno.handle if n == -1
     end
 
     paths.size
@@ -242,7 +242,8 @@ class File < IO
     end
 
     paths.each do |path|
-      POSIX.chown StringValue(path), owner, group
+      n = POSIX.chown Rubinius::Type.coerce_to_path(path), owner, group
+      Errno.handle if n == -1
     end
 
     paths.size
@@ -250,7 +251,9 @@ class File < IO
 
   def chmod(mode)
     mode = Rubinius::Type.coerce_to(mode, Integer, :to_int)
-    POSIX.fchmod @descriptor, clamp_short(mode)
+    n = POSIX.fchmod @descriptor, clamp_short(mode)
+    Errno.handle if n == -1
+    n
   end
 
   def chown(owner, group)
@@ -266,7 +269,9 @@ class File < IO
       group = -1
     end
 
-    POSIX.fchown @descriptor, owner, group
+    n = POSIX.fchown @descriptor, owner, group
+    Errno.handle if n == -1
+    n
   end
 
   ##
@@ -276,6 +281,8 @@ class File < IO
   # by the link). Often not available. Returns number
   # of files in the argument list.
   def self.lchown(owner, group, *paths)
+    raise NotImplementedError, "lchown not implemented on this platform" unless Rubinius::HAVE_LCHOWN
+
     if owner
       owner = Rubinius::Type.coerce_to(owner, Integer, :to_int)
     else
@@ -289,7 +296,8 @@ class File < IO
     end
 
     paths.each do |path|
-      POSIX.lchown StringValue(path), owner_int, group_int
+      n = POSIX.lchown Rubinius::Type.coerce_to_path(path), owner, group
+      Errno.handle if n == -1
     end
 
     paths.size
@@ -313,21 +321,21 @@ class File < IO
     io = Rubinius::Type.try_convert io_or_path, IO, :to_io
 
     if io.is_a? IO
-      Stat.from_fd(io.fileno).directory?
+      Stat.fstat(io.fileno).directory?
     else
       st = Stat.stat io_or_path
       st ? st.directory? : false
     end
   end
 
-  def self.last_nonslash(path,start=nil)
+  def self.last_nonslash(path, start=nil)
     # Find the first non-/ from the right
     data = path.data
     idx = nil
     start ||= (path.size - 1)
 
     start.downto(0) do |i|
-      if data[i] != ?/
+      if data[i] != 47  # ?/
         return i
       end
     end
@@ -343,7 +351,7 @@ class File < IO
   #
   #  File.dirname("/home/gumby/work/ruby.rb")   #=> "/home/gumby/work"
   def self.dirname(path)
-    path = StringValue(path)
+    path = Rubinius::Type.coerce_to_path(path)
 
     # edge case
     return "." if path.empty?
@@ -357,7 +365,7 @@ class File < IO
     if pos = path.find_string_reverse(slash, chunk_size)
       return "/" if pos == 0
 
-      path = path.substring(0, pos)
+      path = path.byteslice(0, pos)
 
       return "/" if path == "/"
 
@@ -369,7 +377,7 @@ class File < IO
       # edge case, only /'s, return /
       return "/" unless idx
 
-      return path.substring(0, idx - 1)
+      return path.byteslice(0, idx - 1)
     end
 
     return "."
@@ -394,9 +402,15 @@ class File < IO
   ##
   # Return true if the named file exists.
   def self.exist?(path)
-    POSIX.stat(StringValue(path), Stat::EXISTS_STRUCT.pointer) == 0
+    st = Stat.stat(path)
+    st ? true : false
   end
 
+  # Pull a constant for Dir local to File so that we don't have to depend
+  # on the global Dir constant working. This sounds silly, I know, but it's a
+  # little bit of defensive coding so Rubinius can run things like fakefs better.
+  PrivateDir = ::Dir
+
   ##
   # Converts a pathname to an absolute pathname. Relative
   # paths are referenced from the current working directory
@@ -409,13 +423,17 @@ class File < IO
   #  File.expand_path("~oracle/bin")           #=> "/home/oracle/bin"
   #  File.expand_path("../../bin", "/tmp/x")   #=> "/bin"
   def self.expand_path(path, dir=nil)
-    path = StringValue(path)
-
+    path = Rubinius::Type.coerce_to_path(path)
+    str = "".force_encoding path.encoding
     first = path[0]
     if first == ?~
       case path[1]
       when ?/
-        path = ENV["HOME"] + path.substring(1, path.size - 1)
+        unless home = ENV["HOME"]
+          raise ArgumentError, "couldn't find HOME environment variable when expanding '~'"
+        end
+
+        path = ENV["HOME"] + path.byteslice(1, path.bytesize - 1)
       when nil
         unless home = ENV["HOME"]
           raise ArgumentError, "couldn't find HOME environment variable when expanding '~'"
@@ -425,24 +443,24 @@ class File < IO
           raise ArgumentError, "HOME environment variable is empty expanding '~'"
         end
 
-        return home
+        return home.dup
       else
-        unless length = path.index("/", 1)
-          length = path.size
+        unless length = path.find_string("/", 1)
+          length = path.bytesize
         end
 
-        name = path.substring 1, length - 1
+        name = path.byteslice 1, length - 1
         unless dir = Rubinius.get_user_home(name)
           raise ArgumentError, "user #{name} does not exist"
         end
 
-        path = dir + path.substring(length, path.size - length)
+        path = dir + path.byteslice(length, path.bytesize - length)
       end
     elsif first != ?/
       if dir
-        dir = File.expand_path dir
+        dir = expand_path dir
       else
-        dir = Dir.pwd
+        dir = PrivateDir.pwd
       end
 
       path = "#{dir}/#{path}"
@@ -450,13 +468,13 @@ class File < IO
 
     items = []
     start = 0
-    size = path.size
+    size = path.bytesize
 
-    while index = path.index("/", start) or (start < size and index = size)
+    while index = path.find_string("/", start) or (start < size and index = size)
       length = index - start
 
       if length > 0
-        item = path.substring start, length
+        item = path.byteslice start, length
 
         if item == ".."
           items.pop
@@ -468,12 +486,13 @@ class File < IO
       start = index + 1
     end
 
-    return "/" if items.empty?
+    if items.empty?
+      str << "/"
+    else
+      items.each { |x| str.append "/#{x}" }
+    end
 
-    str = ""
-    items.each { |x| str.append "/#{x}" }
-
-    return str
+    str
   end
 
   ##
@@ -485,8 +504,8 @@ class File < IO
   #  File.extname("test")            #=> ""
   #  File.extname(".profile")        #=> ""
   def self.extname(path)
-    path = StringValue(path)
-    path_size = path.size
+    path = Rubinius::Type.coerce_to_path(path)
+    path_size = path.bytesize
 
     dot_idx = path.find_string_reverse(".", path_size)
 
@@ -507,7 +526,7 @@ class File < IO
     # last component ends with a .
     return "" if dot_idx == path_size - 1
 
-    return path.substring(dot_idx, path_size - dot_idx)
+    return path.byteslice(dot_idx, path_size - dot_idx)
   end
 
   ##
@@ -523,11 +542,11 @@ class File < IO
   # similar to shell filename globbing. It may contain the
   # following metacharacters:
   #
-  # *:	Matches any file. Can be restricted by other values in the glob. * will match all files; c* will match all files beginning with c; *c will match all files ending with c; and c will match all files that have c in them (including at the beginning or end). Equivalent to / .* /x in regexp.
-  # **:	Matches directories recursively or files expansively.
-  # ?:	Matches any one character. Equivalent to /.{1}/ in regexp.
-  # [set]:	Matches any one character in set. Behaves exactly like character sets in Regexp, including set negation ([^a-z]).
-  # <code></code>:	Escapes the next metacharacter.
+  # *:  Matches any file. Can be restricted by other values in the glob. * will match all files; c* will match all files beginning with c; *c will match all files ending with c; and c will match all files that have c in them (including at the beginning or end). Equivalent to / .* /x in regexp.
+  # **:  Matches directories recursively or files expansively.
+  # ?:  Matches any one character. Equivalent to /.{1}/ in regexp.
+  # [set]:  Matches any one character in set. Behaves exactly like character sets in Regexp, including set negation ([^a-z]).
+  # <code></code>:  Escapes the next metacharacter.
   # flags is a bitwise OR of the FNM_xxx parameters. The same glob pattern and flags are used by Dir::glob.
   #
   #  File.fnmatch('cat',       'cat')        #=> true  : match entire string
@@ -578,7 +597,7 @@ class File < IO
   #  File.fnmatch(pattern, 'a/.b/c/foo', File::FNM_PATHNAME | File::FNM_DOTMATCH) #=> true
   def self.fnmatch(pattern, path, flags=0)
     pattern = StringValue(pattern)
-    path    = StringValue(path)
+    path    = Rubinius::Type.coerce_to_path(path)
     flags   = Rubinius::Type.coerce_to(flags, Fixnum, :to_int)
 
     super pattern, path, flags
@@ -621,9 +640,14 @@ class File < IO
   #   open("d", "w") {}
   #   p File.identical?("a", "d")      #=> false
   def self.identical?(orig, copy)
-    st_o = stat(StringValue(orig))
-    st_c = stat(StringValue(copy))
+    orig = Rubinius::Type.coerce_to_path(orig)
+    st_o = File::Stat.stat(orig)
+    copy = Rubinius::Type.coerce_to_path(copy)
+    st_c = File::Stat.stat(copy)
 
+    return false if st_o.nil? || st_c.nil?
+
+    return false unless st_o.dev == st_c.dev
     return false unless st_o.ino == st_c.ino
     return false unless st_o.ftype == st_c.ftype
     return false unless POSIX.access(orig, Constants::R_OK)
@@ -658,7 +682,7 @@ class File < IO
     else
       # We need to use dup here, since it's possible that
       # StringValue gives us a direct object we shouldn't mutate
-      first = StringValue(first).dup
+      first = Rubinius::Type.coerce_to_path(first).dup
     end
 
     ret = first
@@ -676,7 +700,7 @@ class File < IO
 
         raise ArgumentError, "recursive array" if recursion
       else
-        value = StringValue(el)
+        value = Rubinius::Type.coerce_to_path(el)
       end
 
       if value.prefix? sep
@@ -698,7 +722,7 @@ class File < IO
   #  File.link("testfile", ".testfile")   #=> 0
   #  IO.readlines(".testfile")[0]         #=> "This is line one\n"
   def self.link(from, to)
-    n = POSIX.link StringValue(from), StringValue(to)
+    n = POSIX.link Rubinius::Type.coerce_to_path(from), Rubinius::Type.coerce_to_path(to)
     Errno.handle if n == -1
     n
   end
@@ -723,6 +747,12 @@ class File < IO
     Stat.new(path).mtime
   end
 
+  def self.path(obj)
+    return obj.to_path if obj.respond_to? :to_path
+
+    StringValue(obj)
+  end
+
   ##
   # Returns true if the named file is a pipe.
   def self.pipe?(path)
@@ -754,20 +784,74 @@ class File < IO
   #  File.readlink("link2test")              #=> "testfile"
   def self.readlink(path)
     FFI::MemoryPointer.new(1024) do |ptr|
-      n = POSIX.readlink StringValue(path), ptr, 1024
+      n = POSIX.readlink Rubinius::Type.coerce_to_path(path), ptr, 1024
       Errno.handle if n == -1
 
       return ptr.read_string(n)
     end
   end
 
+  def self.realpath(path, basedir = nil)
+    real = basic_realpath path, basedir
+
+    unless exist? real
+      raise Errno::ENOENT, real
+    end
+
+    real
+  end
+
+  def self.realdirpath(path, basedir = nil)
+    real = basic_realpath path, basedir
+    dir = dirname real
+
+    unless directory? dir
+      raise Errno::ENOENT, real
+    end
+
+    real
+  end
+
+  def self.basic_realpath(path, basedir = nil)
+    path = expand_path(path, basedir || Dir.pwd)
+    real = ''
+    symlinks = {}
+
+    while !path.empty?
+      pos = path.index(SEPARATOR, 1)
+
+      if pos
+        name = path[0...pos]
+        path = path[pos..-1]
+      else
+        name = path
+        path = ''
+      end
+
+      real = join(real, name)
+      if symlink?(real)
+        raise Errno::ELOOP if symlinks[real]
+        symlinks[real] = true
+        if path.empty?
+          path = expand_path(readlink(real), dirname(real))
+        else
+          path = expand_path(join(readlink(real), path), dirname(real))
+        end
+        real = ''
+      end
+    end
+
+    real
+  end
+  private_class_method :basic_realpath
+
   ##
   # Renames the given file to the new name. Raises a SystemCallError
   # if the file cannot be renamed.
   #
   #  File.rename("afile", "afile.bak")   #=> 0
   def self.rename(from, to)
-    n = POSIX.rename StringValue(from), StringValue(to)
+    n = POSIX.rename Rubinius::Type.coerce_to_path(from), Rubinius::Type.coerce_to_path(to)
     Errno.handle if n == -1
     n
   end
@@ -778,7 +862,7 @@ class File < IO
     io = Rubinius::Type.try_convert io_or_path, IO, :to_io
 
     if io.is_a? IO
-      Stat.from_fd(io.fileno).size
+      Stat.fstat(io.fileno).size
     else
       stat(io_or_path).size
     end
@@ -793,7 +877,7 @@ class File < IO
     io = Rubinius::Type.try_convert io_or_path, IO, :to_io
 
     if io.is_a? IO
-      s = Stat.from_fd(io.fileno).size
+      s = Stat.fstat(io.fileno).size
     else
       st = Stat.stat io_or_path
       s = st.size if st
@@ -814,7 +898,7 @@ class File < IO
   #
   #  File.split("/home/gumby/.profile")   #=> ["/home/gumby", ".profile"]
   def self.split(path)
-    p = StringValue(path)
+    p = Rubinius::Type.coerce_to_path(path)
     [dirname(p), basename(p)]
   end
 
@@ -833,7 +917,7 @@ class File < IO
   #
   #  File.symlink("testfile", "link2test")   #=> 0
   def self.symlink(from, to)
-    n = POSIX.symlink StringValue(from), StringValue(to)
+    n = POSIX.symlink Rubinius::Type.coerce_to_path(from), Rubinius::Type.coerce_to_path(to)
     Errno.handle if n == -1
     n
   end
@@ -849,7 +933,7 @@ class File < IO
   ##
   # Copies a file from to to. If to is a directory, copies from to to/from.
   def self.syscopy(from, to)
-    out = File.directory?(to) ? to + File.basename(from) : to
+    out = directory?(to) ? to + basename(from) : to
 
     open(out, 'w') do |f|
       f.write read(from).read
@@ -866,7 +950,7 @@ class File < IO
   #  File.truncate("out", 5)   #=> 0
   #  File.size("out")          #=> 5
   def self.truncate(path, length)
-    path = StringValue(path)
+    path = Rubinius::Type.coerce_to_path(path)
 
     unless exist?(path)
       raise Errno::ENOENT, path
@@ -874,9 +958,7 @@ class File < IO
 
     length = Rubinius::Type.coerce_to length, Integer, :to_int
 
-    n = POSIX.truncate(path, length)
-    Errno.handle if n == -1
-    n
+    prim_truncate(path, length)
   end
 
   ##
@@ -906,7 +988,7 @@ class File < IO
   # See also Dir::rmdir.
   def self.unlink(*paths)
     paths.each do |path|
-      n = POSIX.unlink StringValue(path)
+      n = POSIX.unlink Rubinius::Type.coerce_to_path(path)
       Errno.handle if n == -1
     end
 
@@ -919,6 +1001,8 @@ class File < IO
   # of file names in the argument list.
   #  #=> Integer
   def self.utime(a_in, m_in, *paths)
+    a_in ||= Time.now
+    m_in ||= Time.now
     FFI::MemoryPointer.new(POSIX::TimeVal, 2) do |ptr|
       atime = POSIX::TimeVal.new ptr
       mtime = POSIX::TimeVal.new ptr[1]
@@ -930,12 +1014,33 @@ class File < IO
 
       paths.each do |path|
 
-        n = POSIX.utimes(StringValue(path), ptr)
+        n = POSIX.utimes(Rubinius::Type.coerce_to_path(path), ptr)
         Errno.handle unless n == 0
       end
     end
   end
 
+  def self.world_readable?(path)
+    path = Rubinius::Type.coerce_to_path path
+    return nil unless exists? path
+    mode = Stat.new(path).mode
+    if (mode & Stat::S_IROTH) == Stat::S_IROTH
+      tmp = mode & (Stat::S_IRUGO | Stat::S_IWUGO | Stat::S_IXUGO)
+      return Rubinius::Type.coerce_to tmp, Fixnum, :to_int
+    end
+    nil
+  end
+
+  def self.world_writable?(path)
+    path = Rubinius::Type.coerce_to_path path
+    return nil unless exists? path
+    mode = Stat.new(path).mode
+    if (mode & Stat::S_IWOTH) == Stat::S_IWOTH
+      tmp = mode & (Stat::S_IRUGO | Stat::S_IWUGO | Stat::S_IXUGO)
+      return Rubinius::Type.coerce_to tmp, Fixnum, :to_int
+    end
+  end
+
   ##
   # Returns true if the named file is writable by the effective
   # user id of this process.
@@ -999,12 +1104,56 @@ class File < IO
     alias_method :fnmatch?, :fnmatch
   end
 
+  def initialize(path_or_fd, mode=undefined, perm=undefined, options=undefined)
+    if path_or_fd.kind_of? Integer
+      super(path_or_fd, mode, options)
+      @path = nil
+    else
+      path = Rubinius::Type.coerce_to_path path_or_fd
+
+      # TODO: fix normalize_options
+      case mode
+      when String, Fixnum
+        # do nothing
+      when nil, undefined
+        mode = "r"
+      when Hash
+        options = mode
+        mode = nil
+      else
+        options = Rubinius::Type.coerce_to mode, Hash, :to_hash
+        mode = nil
+      end
+
+      if undefined.equal?(options)
+        options = Rubinius::Type.try_convert(perm, Hash, :to_hash)
+        perm = undefined if options
+      end
+
+      nmode, binary, external, internal = IO.normalize_options(mode, options)
+      nmode ||= "r"
+
+      perm = 0666 if undefined.equal? perm
+
+      fd = IO.sysopen(path, nmode, perm)
+      Errno.handle path if fd < 0
+
+      @path = path
+      super(fd, mode, options)
+    end
+  end
+
+  private :initialize
+
   def atime
     Stat.new(@path).atime
   end
 
   def reopen(other, mode = 'r+')
     rewind unless closed?
+    unless other.kind_of? IO
+      other = Rubinius::Type.coerce_to_path(other)
+    end
     super(other, mode)
   end
 
@@ -1030,9 +1179,11 @@ class File < IO
   end
 
   def stat
-    Stat.from_fd @descriptor
+    Stat.fstat @descriptor
   end
 
+  alias_method :to_path, :path
+
   def truncate(length)
     length = Rubinius::Type.coerce_to length, Integer, :to_int
 
@@ -1041,9 +1192,7 @@ class File < IO
 
     flush
     reset_buffering
-    n = POSIX.ftruncate(@descriptor, length)
-    Errno.handle if n == -1
-    n
+    prim_ftruncate(length)
   end
 
   def inspect
@@ -1051,6 +1200,11 @@ class File < IO
     return_string << " (closed)" if closed?
     return_string << ">"
   end
+
+  def size
+    raise IOError, "closed stream" if closed?
+    stat.size
+  end
 end     # File
 
 # Inject the constants into IO
@@ -1058,332 +1212,21 @@ class IO
   include File::Constants
 end
 
+File::Stat = Rubinius::Stat
 class File::Stat
-  class Struct < FFI::Struct
-    config "rbx.platform.stat", :st_dev, :st_ino, :st_mode, :st_nlink,
-           :st_uid, :st_gid, :st_rdev, :st_size, :st_blksize, :st_blocks,
-           :st_atime, :st_mtime, :st_ctime
-  end
+  @module_name = :"File::Stat"
 
-  EXISTS_STRUCT = Struct.new
-
-  include Comparable
-
-  S_IRUSR  = Rubinius::Config['rbx.platform.file.S_IRUSR']
-  S_IWUSR  = Rubinius::Config['rbx.platform.file.S_IWUSR']
-  S_IXUSR  = Rubinius::Config['rbx.platform.file.S_IXUSR']
-  S_IRGRP  = Rubinius::Config['rbx.platform.file.S_IRGRP']
-  S_IWGRP  = Rubinius::Config['rbx.platform.file.S_IWGRP']
-  S_IXGRP  = Rubinius::Config['rbx.platform.file.S_IXGRP']
-  S_IROTH  = Rubinius::Config['rbx.platform.file.S_IROTH']
-  S_IWOTH  = Rubinius::Config['rbx.platform.file.S_IWOTH']
-  S_IXOTH  = Rubinius::Config['rbx.platform.file.S_IXOTH']
-
-  S_IFMT   = Rubinius::Config['rbx.platform.file.S_IFMT']
-  S_IFIFO  = Rubinius::Config['rbx.platform.file.S_IFIFO']
-  S_IFCHR  = Rubinius::Config['rbx.platform.file.S_IFCHR']
-  S_IFDIR  = Rubinius::Config['rbx.platform.file.S_IFDIR']
-  S_IFBLK  = Rubinius::Config['rbx.platform.file.S_IFBLK']
-  S_IFREG  = Rubinius::Config['rbx.platform.file.S_IFREG']
-  S_IFLNK  = Rubinius::Config['rbx.platform.file.S_IFLNK']
-  S_IFSOCK = Rubinius::Config['rbx.platform.file.S_IFSOCK']
-  S_IFWHT  = Rubinius::Config['rbx.platform.file.S_IFWHT']
-  S_ISUID  = Rubinius::Config['rbx.platform.file.S_ISUID']
-  S_ISGID  = Rubinius::Config['rbx.platform.file.S_ISGID']
-  S_ISVTX  = Rubinius::Config['rbx.platform.file.S_ISVTX']
-
-  POSIX    = FFI::Platform::POSIX
-
-  attr_reader :path
-
-  def self.create(path)
-    path = StringValue path
-    stat = allocate
-    Rubinius.privately { stat.setup path, Struct.new }
-  end
-
-  def self.stat(path)
-    stat = create path
-
-    result = POSIX.stat stat.path, stat.pointer
-    return nil unless result == 0
-
-    stat
-  end
-
-  # --
-  # Stat.lstat raises whereas Stat.stat does not because most things
-  # that use Stat.stat do not expect exceptions but most things that
-  # uses Stat.lstat do.
-  # ++
-  def self.lstat(path)
-    stat = create path
-
-    result = POSIX.lstat stat.path, stat.pointer
-    Errno.handle path unless result == 0
-
-    stat
-  end
-
-  ##
-  # File::Stat#from_fd is used to support IO#stat which does not necessarily
-  # have a path.
-
-  def self.from_fd(descriptor)
-    stat = allocate
-    struct = Struct.new
-
-    result = POSIX.fstat descriptor, struct.pointer
-    Errno.handle "file descriptor #{descriptor}" unless result == 0
-
-    Rubinius.privately { stat.setup nil, struct }
-  end
-
-  def initialize(path)
-    @path = StringValue path
-    @stat = Struct.new
-    result = POSIX.stat @path, @stat.pointer
-    Errno.handle path unless result == 0
-  end
-
-  private :initialize
-
-  def setup(path, struct)
-    @path = path
-    @stat = struct
-    self
-  end
-
-  private :setup
-
-  def pointer
-    @stat.pointer
-  end
-
-  def atime
-    Time.at @stat[:st_atime]
-  end
-
-  def blksize
-    @stat[:st_blksize]
-  end
-
-  def blocks
-    @stat[:st_blocks]
-  end
-
-  def blockdev?
-    @stat[:st_mode] & S_IFMT == S_IFBLK
-  end
-
-  def chardev?
-    @stat[:st_mode] & S_IFMT == S_IFCHR
-  end
-
-  def ctime
-    Time.at @stat[:st_ctime]
-  end
-
-  def dev
-    @stat[:st_dev]
-  end
-
-  def dev_major
-    major = POSIX.major @stat[:st_dev]
-    major < 0 ? nil : major
-  end
-
-  def dev_minor
-    minor = POSIX.major @stat[:st_dev]
-    minor < 0 ? nil : minor
-  end
-
-  def directory?
-    @stat[:st_mode] & S_IFMT == S_IFDIR
-  end
-
-  def executable?
-    return true if superuser?
-    return @stat[:st_mode] & S_IXUSR != 0 if owned?
-    return @stat[:st_mode] & S_IXGRP != 0 if grpowned?
-    return @stat[:st_mode] & S_IXOTH != 0
-  end
-
-  def executable_real?
-    return true if rsuperuser?
-    return @stat[:st_mode] & S_IXUSR != 0 if rowned?
-    return @stat[:st_mode] & S_IXGRP != 0 if rgrpowned?
-    return @stat[:st_mode] & S_IXOTH != 0
-  end
-
-  def file?
-    @stat[:st_mode] & S_IFMT == S_IFREG
-  end
-
-  def ftype
-    if file?
-      "file"
-    elsif directory?
-      "directory"
-    elsif chardev?
-      "characterSpecial"
-    elsif blockdev?
-      "blockSpecial"
-    elsif pipe?
-      "fifo"
-    elsif socket?
-      "socket"
-    elsif symlink?
-      "link"
-    else
-      "unknown"
+  def world_readable?
+    if mode & S_IROTH == S_IROTH
+      tmp = mode & (S_IRUGO | S_IWUGO | S_IXUGO)
+      return Rubinius::Type.coerce_to tmp, Fixnum, :to_int
     end
   end
 
-  def gid
-    @stat[:st_gid]
+  def world_writable?
+    if mode & S_IWOTH == S_IWOTH
+      tmp = mode & (S_IRUGO | S_IWUGO | S_IXUGO)
+      return Rubinius::Type.coerce_to tmp, Fixnum, :to_int
+    end
   end
-
-  def grpowned?
-    @stat[:st_gid] == POSIX.getegid
-  end
-
-  def ino
-    @stat[:st_ino]
-  end
-
-  def inspect
-    "#<File::Stat dev=0x#{self.dev.to_s(16)}, ino=#{self.ino}, " \
-    "mode=#{sprintf("%07d", self.mode.to_s(8).to_i)}, nlink=#{self.nlink}, " \
-    "uid=#{self.uid}, gid=#{self.gid}, rdev=0x#{self.rdev.to_s(16)}, " \
-    "size=#{self.size}, blksize=#{self.blksize}, blocks=#{self.blocks}, " \
-    "atime=#{self.atime}, mtime=#{self.mtime}, ctime=#{self.ctime}>"
-  end
-
-  def nlink
-    @stat[:st_nlink]
-  end
-
-  def mtime
-    Time.at @stat[:st_mtime]
-  end
-
-  def mode
-    @stat[:st_mode]
-  end
-
-  def owned?
-    @stat[:st_uid] == POSIX.geteuid
-  end
-
-  def path
-    @path
-  end
-
-  def pipe?
-    @stat[:st_mode] & S_IFMT == S_IFIFO
-  end
-
-  def rdev
-    @stat[:st_rdev]
-  end
-
-  def rdev_major
-    major = POSIX.major @stat[:st_rdev]
-    major < 0 ? nil : major
-  end
-
-  def rdev_minor
-    minor = POSIX.minor @stat[:st_rdev]
-    minor < 0 ? nil : minor
-  end
-
-  def readable?
-    return true if superuser?
-    return @stat[:st_mode] & S_IRUSR != 0 if owned?
-    return @stat[:st_mode] & S_IRGRP != 0 if grpowned?
-    return @stat[:st_mode] & S_IROTH != 0
-  end
-
-  def readable_real?
-    return true if rsuperuser?
-    return @stat[:st_mode] & S_IRUSR != 0 if rowned?
-    return @stat[:st_mode] & S_IRGRP != 0 if rgrpowned?
-    return @stat[:st_mode] & S_IROTH != 0
-  end
-
-  def setgid?
-    @stat[:st_mode] & S_ISGID != 0
-  end
-
-  def setuid?
-    @stat[:st_mode] & S_ISUID != 0
-  end
-
-  def sticky?
-    @stat[:st_mode] & S_ISVTX != 0
-  end
-
-  def size
-    @stat[:st_size]
-  end
-
-  def size?
-    size == 0 ? nil : size
-  end
-
-  def socket?
-    @stat[:st_mode] & S_IFMT == S_IFSOCK
-  end
-
-  def symlink?
-    @stat[:st_mode] & S_IFMT == S_IFLNK
-  end
-
-  def uid
-    @stat[:st_uid]
-  end
-
-  def writable?
-    return true if superuser?
-    return @stat[:st_mode] & S_IWUSR != 0 if owned?
-    return @stat[:st_mode] & S_IWGRP != 0 if grpowned?
-    return @stat[:st_mode] & S_IWOTH != 0
-  end
-
-  def writable_real?
-    return true if rsuperuser?
-    return @stat[:st_mode] & S_IWUSR != 0 if rowned?
-    return @stat[:st_mode] & S_IWGRP != 0 if rgrpowned?
-    return @stat[:st_mode] & S_IWOTH != 0
-  end
-
-  def zero?
-    @stat[:st_size] == 0
-  end
-
-  def <=>(other)
-    return nil unless other.is_a?(File::Stat)
-    self.mtime <=> other.mtime
-  end
-
-  def rgrpowned?
-    @stat[:st_gid] == POSIX.getgid
-  end
-  private :rgrpowned?
-
-  def rowned?
-    @stat[:st_uid] == POSIX.getuid
-  end
-  private :rowned?
-
-  def rsuperuser?
-    POSIX.getuid == 0
-  end
-  private :rsuperuser?
-
-  def superuser?
-    POSIX.geteuid == 0
-  end
-  private :superuser?
-end     # File::Stat
+end
