@@ -31,17 +31,35 @@ class Fixnum < Integer
   alias_method :modulo, :%
 
   def fdiv(n)
-    to_f / n
+    if n.kind_of?(Fixnum)
+      to_f / n
+    else
+      redo_coerced :fdiv, n
+    end
   end
 
-  alias_method :quo, :fdiv
-
-  # taint and untaint are noops on Fixnum
-  def taint
-    self
+  def imaginary
+    0
   end
 
-  def untaint
-    self
+  # Must be it's own method, so that super calls the correct method
+  # on Numeric
+  def div(o)
+    if o.is_a?(Float) && o == 0.0
+      raise ZeroDivisionError, "division by zero"
+    end
+    divide(o).floor
+  end
+
+  def **(o)
+    Rubinius.primitive :fixnum_pow
+
+    if o.is_a?(Float) && self < 0 && o != o.round
+      return Complex.new(self, 0) ** o
+    elsif o.is_a?(Integer) && o < 0
+      return Rational.new(self, 1) ** o
+    end
+
+    redo_coerced :**, o
   end
 end
