@@ -52,7 +52,13 @@ class Float
       end
     end
 
-    Rubinius::Type.binary_string("f#{ms.serialize_integer(str.length)}#{str}")
+    sl = str.length
+    if sign == 1
+      ss = "-"
+      sl += 1
+    end
+
+    Rubinius::Type.binary_string("f#{ms.serialize_integer(sl)}#{ss}#{str}")
   end
 end
 
@@ -81,7 +87,7 @@ class Time
     # Order matters.
     extra_values = {}
     extra_values[:offset] = gmt_offset unless gmt?
-    extra_values[:zone] = zone.encode(Encoding.find('locale'))
+    extra_values[:zone] = zone
 
     if nsec > 0
       # MRI serializes nanoseconds as a Rational using an
@@ -257,8 +263,7 @@ end
 
 class Struct
   def __marshal__(ms)
-    attr_syms = _attrs.map { |a| "@#{a}".to_sym }
-    exclude = Rubinius::Type.convert_to_names attr_syms
+    exclude = _attrs.map { |a| "@#{a}".to_sym }
 
     out =  ms.serialize_instance_variables_prefix(self, exclude)
     out << ms.serialize_extended_object(self)
@@ -301,11 +306,10 @@ class Hash
   def __marshal__(ms)
     raise TypeError, "can't dump hash with default proc" if default_proc
 
-    exclude_syms = %w[
+    excluded_ivars = %w[
       @capacity @mask @max_entries @size @entries @default_proc @default
       @state @compare_by_identity @head @tail @table
     ].map { |a| a.to_sym }
-    excluded_ivars = Rubinius::Type.convert_to_names exclude_syms
 
     out =  ms.serialize_instance_variables_prefix(self, excluded_ivars)
     out << ms.serialize_extended_object(self)
@@ -1020,7 +1024,17 @@ module Marshal
 
     def serialize_symbol(obj)
       str = obj.to_s
-      Rubinius::Type.binary_string(":#{serialize_integer(str.bytesize)}#{str}")
+      mf = "I" unless str.ascii_only?
+      if mf
+        if Rubinius::Type.object_encoding(obj).equal? Encoding::BINARY
+          me = serialize_integer(0)
+        elsif serialize_encoding?(obj)
+          me = serialize_integer(1) + serialize_encoding(obj.encoding)
+        end
+      end
+      mi = serialize_integer(str.bytesize)
+      s = Rubinius::Type.binary_string str
+      Rubinius::Type.binary_string("#{mf}:#{mi}#{s}#{me}")
     end
 
     def serialize_string(str)
