@@ -33,36 +33,13 @@ class Module
     module_eval(&block) if block
   end
 
-  def const_get(name, inherit = true)
-    Rubinius::Type.const_get(self, name, inherit)
+  def const_get(name, inherit=true)
+    Rubinius::Type.const_get self, name, inherit
   end
 
-  def const_defined?(name, search_parents=true)
-    name = Rubinius::Type.coerce_to_constant_name name
-    return true if @constant_table.has_name? name
-
-    # a silly special case
-    if self.equal? Object
-      mod = direct_superclass
-      while mod.kind_of? Rubinius::IncludedModule
-        return true if mod.constant_table.has_name? name
-        mod = mod.direct_superclass
-      end
-    end
-
-    if search_parents
-      current = self.direct_superclass
-      while current
-        return true if current.constant_table.has_name? name
-        current = current.direct_superclass
-      end
-
-      if instance_of?(Module)
-        return true if Object.constant_table.has_name? name
-      end
-    end
-
-    return false
+  def const_defined?(name, inherit=true)
+    c = Rubinius::Type.const_get self, name, inherit, false
+    return c.equal?(undefined) ? false : true
   end
 
   def attr(*attributes)
@@ -126,10 +103,6 @@ class Module
   def constants(all=undefined)
     tbl = Rubinius::LookupTable.new
 
-    # When calling `constants` on "normal" classes the result does not include
-    # their own class name. BasicObject is a special case in this regard.
-    tbl[:BasicObject] = true if self == BasicObject
-
     @constant_table.each do |name, constant, visibility|
       tbl[name] = true unless visibility == :private
     end
@@ -153,7 +126,7 @@ class Module
       end
     end
 
-    Rubinius::Type.convert_to_names tbl.keys
+    tbl.keys
   end
 
   def private_constant(*names)
@@ -302,10 +275,7 @@ class Module
 
     raise PrimitiveFailure, "Module#__class_variables__ primitive failed"
   end
-
-  def class_variables
-    Rubinius::Type.convert_to_names(__class_variables__)
-  end
+  alias_method :class_variables, :__class_variables__
 
   def name
     Rubinius::Type.module_name self
@@ -401,19 +371,19 @@ class Module
   end
 
   def public_method_defined?(sym)
-    sym = Rubinius::Type.coerce_to_symbol(sym)
+    sym = Rubinius::Type.coerce_to_reflection_name(sym)
     mod, meth = lookup_method(sym, false)
     meth ? meth.public? : false
   end
 
   def private_method_defined?(sym)
-    sym = Rubinius::Type.coerce_to_symbol(sym)
+    sym = Rubinius::Type.coerce_to_reflection_name(sym)
     mod, meth = lookup_method(sym, false)
     meth ? meth.private? : false
   end
 
   def protected_method_defined?(sym)
-    sym = Rubinius::Type.coerce_to_symbol(sym)
+    sym = Rubinius::Type.coerce_to_reflection_name(sym)
     mod, meth = lookup_method(sym, false)
     meth ? meth.protected? : false
   end
@@ -447,7 +417,7 @@ class Module
       end
     end
 
-    Rubinius::Type.convert_to_names ary
+    ary
   end
 
   def public_instance_methods(all=true)
@@ -484,7 +454,7 @@ class Module
       end
     end
 
-    Rubinius::Type.convert_to_names ary
+    ary
   end
 
   private :filter_methods
@@ -644,6 +614,14 @@ class Module
 
     if Rubinius::Type.object_kind_of? value, Module
       Rubinius::Type.set_module_name value, name, self
+
+      if Object.equal? self
+        value.constant_table.each do |n, c, _|
+          if Rubinius::Type.object_kind_of? c, Module and !c.name
+            Rubinius::Type.set_module_name c, n, value
+          end
+        end
+      end
     end
 
     @constant_table.store(name, value, :public)
@@ -836,7 +814,7 @@ class Module
   end
 
   def dynamic_method(name, file="(dynamic)", line=1)
-    g = Rubinius::ToolSet::Runtime::Generator.new
+    g = Rubinius::ToolSets::Runtime::Generator.new
     g.name = name.to_sym
     g.file = file.to_sym
     g.set_line Integer(line)
