@@ -3,6 +3,10 @@ module Signal
     "EXIT" => 0
   }
 
+  Numbers = {
+    0 => "EXIT"
+  }
+
   @threads = {}
   @handlers = {}
 
@@ -25,6 +29,7 @@ module Signal
 
     # If no command, use the block.
     prc ||= block
+    prc = prc.to_s if prc.kind_of?(Symbol)
 
     case prc
     when "DEFAULT", "SIG_DFL"
@@ -32,12 +37,14 @@ module Signal
       old = @handlers.delete(number)
 
       if number != Names["EXIT"]
-        Rubinius.watch_signal(-number)
+        Rubinius.watch_signal(-number, prc.nil? || prc == 'IGNORE')
       end
 
       return "DEFAULT" unless had_old
-      return old ? old : "IGNORE"
-    when "IGNORE", "SIG_IGN", nil
+      return old ? old : nil
+    when "IGNORE", "SIG_IGN"
+      prc = "IGNORE"
+    when nil
       prc = nil
     when "EXIT"
       prc = proc { exit }
@@ -55,11 +62,11 @@ module Signal
     @handlers[number] = prc
 
     if number != Names["EXIT"]
-      Rubinius.watch_signal number
+      Rubinius.watch_signal(number, prc.nil? || prc == 'IGNORE')
     end
 
     return "DEFAULT" unless had_old
-    return old ? old : "IGNORE"
+    return old ? old : nil
   end
 
   def self.run_handler(sig)
@@ -72,5 +79,4 @@ module Signal
   def self.list
     Names.dup
   end
-
 end
