@@ -233,10 +233,10 @@ class String
 
     table = count_table(*strings).__data__
 
-    count = i = 0
-    while i < @num_bytes
-      count += 1 if table[@data[i]] == 1
-      i += 1
+    count = bytes = 0
+    while bytes < @num_bytes
+      count += 1 if table[@data[bytes]] == 1
+      bytes += find_character(bytes).num_bytes
     end
 
     count
@@ -304,11 +304,12 @@ class String
 
   def each_char
     return to_enum :each_char unless block_given?
-    i = 0
-    n = size
-    while i < n
-      yield substring(i, 1)
-      i += 1
+
+    bytes = 0
+    while bytes < @num_bytes
+      char = find_character(bytes)
+      yield char
+      bytes += char.num_bytes
     end
 
     self
@@ -939,7 +940,7 @@ class String
       options = Rubinius::Type.coerce_to options, Hash, :to_hash
     end
 
-    if ascii_only? and from_enc.ascii_compatible? and to_enc.ascii_compatible?
+    if ascii_only? and from_enc.ascii_compatible? and to_enc and to_enc.ascii_compatible?
       force_encoding to_enc
     elsif to_enc and from_enc != to_enc
       ec = Encoding::Converter.new from_enc, to_enc, options
@@ -963,6 +964,10 @@ class String
       else
         raise ArgumentError, "unexpected value for xml option: #{xml.inspect}"
       end
+
+      if options[:universal_newline]
+        gsub!(/\r\n|\r/, "\r\n" => "\n", "\r" => "\n")
+      end
     end
 
     self
@@ -1489,7 +1494,6 @@ class String
       chr = chr_at bytes
       return unless chr.ascii?
 
-      code = chr.ord
       case chr.ord
       when 13
         # do nothing
@@ -1559,6 +1563,7 @@ class String
     self.num_bytes = other.num_bytes
     @hash_value = nil
     force_encoding(other.encoding)
+    @valid_encoding = other.valid_encoding?
 
     Rubinius::Type.infect(self, other)
   end
@@ -1909,6 +1914,57 @@ class String
     result
   end
 
+  # Removes invalid byte sequences from a String, available since Ruby 2.1.
+  def scrub(replace = nil)
+    output = ''
+    input  = dup
+
+    # The default replacement character is the "Unicode replacement" character.
+    # (U+FFFD).
+    if !replace and !block_given?
+      replace = "\xEF\xBF\xBD".force_encoding("UTF-8")
+        .encode(self.encoding, :undef => :replace, :replace => '?')
+    end
+
+    if replace
+      unless replace.is_a?(String)
+        raise(
+          TypeError,
+          "no implicit conversion of #{replace.class} into String"
+        )
+      end
+
+      unless replace.valid_encoding?
+        raise(
+          ArgumentError,
+          "replacement must be a valid byte sequence '#{replace.inspect}'"
+        )
+      end
+
+      replace = replace.force_encoding(Encoding::BINARY)
+    end
+
+    converter = Encoding::Converter.new(input.encoding, Encoding::BINARY)
+
+    while input.length > 0
+      result = converter.primitive_convert(input, output, output.length)
+
+      if result == :finished
+        break
+      elsif result == :undefined_conversion
+        output << converter.primitive_errinfo[3]
+      else
+        if block_given?
+          output << yield(converter.primitive_errinfo[3])
+        else
+          output << replace
+        end
+      end
+    end
+
+    return output.force_encoding(encoding)
+  end
+
   def []=(index, count_or_replacement, replacement=undefined)
     if undefined.equal?(replacement)
       replacement = count_or_replacement
@@ -2314,109 +2370,106 @@ class String
     source = StringValue(source).dup
     replacement = StringValue(replacement).dup
 
-    self.modify!
-
-    return self.delete!(source) if replacement.empty?
+    return delete!(source) if replacement.empty?
     return if @num_bytes == 0
 
     invert = source[0] == ?^ && source.length > 1
-    if invert
-      source.slice!(0)
-    end
-    expanded = source.tr_expand! nil, true
-    size = source.size
-    src = source.__data__
+
+    source.slice!(0) if invert
+    source.tr_expand! nil, true
+    replacement.tr_expand! nil, false
+
+    multi_table = {}
 
     if invert
-      replacement.tr_expand! nil, false
-      r = replacement.__data__[replacement.size-1]
+      r = replacement.__data__[replacement.size - 1]
       table = Rubinius::Tuple.pattern 256, r
-      multi_table = nil
 
       source.each_char do |chr|
         if chr.bytesize > 1
-          multi_table ||= Hash.new { r }
           multi_table[chr] = -1
         else
           table[chr.ord] = -1
         end
       end
     else
-      table = Rubinius::Tuple.pattern 256, -1
-      multi_table = nil
-
-      replacement.tr_expand! nil, false
       repl = replacement.__data__
       rsize = replacement.size
+      table = Rubinius::Tuple.pattern 256, -1
+
       i = 0
       source.each_char do |chr|
         repl_char = replacement[i]
+
         if repl_char && (chr.bytesize > 1 || repl_char.bytesize > 1)
-          multi_table ||= {}
           multi_table[chr] = repl_char
         else
           r = repl[i] if i < rsize
           table[chr.ord] = r
         end
+
         i += 1
       end
     end
 
-    self.modify!
+    destination = dup
     modified = false
 
     if squeeze
       last = nil
+      byte_size = 0
 
       i = 0
-      byte_size = 0
       each_char do |chr|
         c = -1
-        if chr.bytesize == 1
-          c = table[chr.ord]
-        end
+        c = table[chr.ord] if chr.bytesize == 1
+
         if c >= 0
           c_char = c.chr
           next if last == c_char
           byte_size += 1
-          self[i] = c_char
+          destination[i] = c_char
           last = c_char
           modified = true
-        elsif multi_table && (c = multi_table[chr])
+        elsif c = multi_table[chr]
           next if last == c
-          self[i] = c
+          destination[i] = c
           last = c
           modified = true
           byte_size += c.bytesize
         else
-          self[i] = chr
+          destination[i] = chr
           byte_size += chr.bytesize
           last = nil
         end
+
         i += 1
       end
 
-      self.num_bytes = byte_size if byte_size < @num_bytes
+      destination.num_bytes = byte_size if byte_size < @num_bytes
     else
       i = 0
       each_char do |chr|
         c = -1
-        if chr.bytesize == 1
-          c = table[chr.ord]
-        end
+        c = table[chr.ord] if chr.bytesize == 1
+
         if c >= 0
           c_char = c.chr
-          self[i] = c_char
+          destination[i] = c_char
           modified = true
-        elsif multi_table && (c = multi_table[chr])
-          self[i] = c
+        elsif c = multi_table[chr]
+          destination[i] = c
           modified = true
         end
         i += 1
       end
     end
 
-    return modified ? self : nil
+    if modified
+      replace(destination)
+    else
+      nil
+    end
   end
 
   def <=>(other)
