@@ -7,14 +7,17 @@ module Math
     :atan,
     :atan2,
     :atanh,
+    :cbrt,
     :cos,
     :cosh,
     :erf,
     :erfc,
     :exp,
     :frexp,
+    :gamma,
     :hypot,
     :ldexp,
+    :lgamma,
     :log,
     :log10,
     :log2,
