@@ -1,7 +1,6 @@
 class Thread
-  Thread.current.setup(true)
   Thread.initialize_main_thread(Thread.current)
-  dg = ThreadGroup.new
+  dg = ThreadGroup::Default
   Default = dg
   dg.add Thread.current
 end
