@@ -651,40 +651,27 @@ to rebuild the compiler.
 
     #Check Ruby syntax of source
     def check_syntax
-      parser = Rubinius::ToolSet::Runtime::Melbourne
+      parser = Rubinius::ToolSets::Runtime::Melbourne
 
       if @script
         if File.exists?(@script)
           mel = parser.new @script, 1, []
-
-          begin
-            mel.parse_file
-          rescue SyntaxError => e
-            show_syntax_errors(mel.syntax_errors)
-            exit 1
-          end
+          mel.parse_file
         else
           puts "rbx: Unable to find file -- #{@script} (LoadError)"
           exit 1
         end
       elsif not @evals.empty?
-        begin
-          mel = parser.parse_string @evals.join("\n")
-        rescue SyntaxError => e
-          show_syntax_errors(mel.syntax_errors)
-          exit 1
-        end
+        mel = parser.parse_string @evals.join("\n")
       else
-        begin
-          mel = parser.parse_string STDIN.read
-        rescue SyntaxError => e
-          show_syntax_errors(mel.syntax_errors)
-          exit 1
-        end
+        mel = parser.parse_string STDIN.read
       end
 
       puts "Syntax OK"
       exit 0
+    rescue SyntaxError => e
+      show_syntax_errors(mel.syntax_errors)
+      exit 1
     end
 
     # Run IRB unless we were passed -e, -S arguments or a script to run.
@@ -717,6 +704,11 @@ to rebuild the compiler.
       end
     end
 
+    def flush_stdio
+      STDOUT.flush
+      STDERR.flush
+    end
+
     # Cleanup and at_exit processing.
     def epilogue
       @stage = "running at_exit handlers"
@@ -733,6 +725,8 @@ to rebuild the compiler.
       ::GC.start
       ObjectSpace.run_finalizers
 
+      flush_stdio
+
       # TODO: Fix these with better -X processing
       if Config['rbx.jit_stats']
         p VM.jit_info
