@@ -1,9 +1,11 @@
 ##
 # Platform specific behavior for Math.
 
+module Rubinius
 module FFI::Platform::Math
   extend FFI::Library
 
+  attach_function :fabs,  [:double], :double
   attach_function :atan2, [:double, :double], :double
   attach_function :cos,   [:double], :double
   attach_function :sin,   [:double], :double
@@ -26,6 +28,9 @@ module FFI::Platform::Math
   attach_function :hypot, [:double, :double], :double
   attach_function :erf,   [:double], :double
   attach_function :erfc,  [:double], :double
+  attach_function :cbrt, [:double], :double
+  attach_function :tgamma, [:double], :double
+  attach_function :lgamma_r, [:double, :pointer], :double
 
   # Rubinius-specific, used in Marshal
   attach_function :modf,  [:double, :pointer], :double
@@ -38,3 +43,4 @@ module FFI::Platform::Math
     log10(x) / log10(2.0)
   end
 end
+end
