@@ -1,6 +1,6 @@
 # -*- encoding: us-ascii -*-
 
-module Rubinius::ToolSet.current::TS
+module Rubinius::ToolSets.current::ToolSet
   module AST
     class EndData < Node
       attr_accessor :data
