@@ -1,6 +1,6 @@
 # -*- encoding: us-ascii -*-
 
-module Rubinius::ToolSet.current::TS
+module Rubinius::ToolSets.current::ToolSet
   module AST
     class Send < Node
       attr_accessor :receiver, :name, :privately, :block, :variable, :vcall_style
@@ -108,22 +108,19 @@ module Rubinius::ToolSet.current::TS
       end
 
       def arguments_sexp
-        return nil if @vcall_style
-
         sexp = [:arglist]
-        sexp << @block.to_sexp if @block.kind_of? BlockPass
+        sexp << @block.to_sexp if @block
         sexp
       end
 
       def to_sexp
-        sexp = [sexp_name, receiver_sexp, @name, arguments_sexp]
         case @block
         when For
-          @block.to_sexp.insert 1, @receiver.to_sexp
-        when Iter
-          @block.to_sexp.insert 1, sexp
+          n, x, b = @block.to_sexp
+          xs = @receiver.to_sexp
+          [n, x, xs, b]
         else
-          sexp
+          [sexp_name, receiver_sexp, @name, arguments_sexp]
         end
       end
     end
@@ -863,6 +860,10 @@ module Rubinius::ToolSet.current::TS
       def splat_index
         @splat
       end
+
+      def to_sexp
+        @arguments.to_sexp
+      end
     end
 
     class For19 < For
@@ -1008,7 +1009,14 @@ module Rubinius::ToolSet.current::TS
       end
 
       def to_sexp
-        arguments_sexp :yield
+        args = @arguments.to_sexp
+        args << @block.to_sexp if @block
+        if @argument_count == 1 and !@yield_splat and @arguments.splat.nil? and
+           @arguments.array.first.kind_of? SplatValue
+          [:yield, [:array] + args]
+        else
+          [:yield] + args
+        end
       end
     end
 
