@@ -1,5 +1,5 @@
 #define RBX_PROGRAM_NAME     "rbx"
-#define RBX_HOST             "amd64-unknown-openbsd5.5"
+#define RBX_HOST             "amd64-unknown-openbsd5.8"
 #define RBX_CPU              "amd64"
 #define RBX_VENDOR           "unknown"
 #define RBX_OS               "openbsd"
@@ -9,15 +9,17 @@
 #define RBX_LDSHAREDXX       "g++ -shared -lstdc++ -fPIC"
 #define RBX_SIZEOF_LONG      8
 #define RBX_LLVM_API_VER     302
-#define RBX_LIBC             "libc.so.73"
+#define RBX_LIBC             "/usr/lib/libc.so.80"
 #define RBX_HAVE_LCHMOD      false
 #define RBX_HAVE_LCHOWN      true
+#define RBX_DEBUG_BUILD      false
 #define RBX_LITTLE_ENDIAN    1
 #define RBX_HAVE_TR1         1
 #define RBX_HAVE_TR1_HASH    1
 #define RBX_BSD           1
 #define RBX_FIBER_ENABLED    1
 
+#define HAS_EXECINFO         1
 #define HAVE_SPT_REUSEARGV   1
 #define HAVE_CLOCK_GETTIME   1
 #define HAVE_NL_LANGINFO     1
@@ -26,6 +28,7 @@
 #define HAVE_TM_ZONE         1
 #define HAVE_TIMEZONE        1
 #define HAVE_TZNAME          1
+#define HAVE_DAYLIGHT        1
 #define HAVE_STRING_H        1
 #define HAVE_SYS_TIME_H      1
 #define HAVE_SYS_TIMES_H     1
